@@ -691,7 +691,7 @@ AMF_RESULT HevcParser::QueryOutput(amf::AMFData** ppData)
     {
         return AMF_OK;
     }
-    if((m_bEof && m_ReadData.GetSize() == 0) || m_maxFramesNumber && m_PacketCount >= m_maxFramesNumber)
+    if((m_bEof && m_ReadData.GetSize() == 0) || ((m_maxFramesNumber > 0) && (m_PacketCount >= m_maxFramesNumber)))
     {
         return AMF_EOF;
     }
