@@ -393,7 +393,7 @@ AMF_RESULT ParamConverterTransferCharacteristic(const std::wstring& value, amf::
 		paramValue = AMF_COLOR_TRANSFER_CHARACTERISTIC_IEC61966_2_1;
 	} else if (uppValue == L"BT2020_10" || uppValue == L"14") {
 		paramValue = AMF_COLOR_TRANSFER_CHARACTERISTIC_BT2020_10;
-	} else if (uppValue == L"BT2020_10" || uppValue == L"15") {
+	} else if (uppValue == L"BT2020_12" || uppValue == L"15") {
 		paramValue = AMF_COLOR_TRANSFER_CHARACTERISTIC_BT2020_12;
 	} else if (uppValue == L"SMPTE2084" || uppValue == L"16") {
 		paramValue = AMF_COLOR_TRANSFER_CHARACTERISTIC_SMPTE2084;
@@ -878,38 +878,6 @@ AMF_RESULT ParamConverterFRCProfile(const std::wstring& value, amf::AMFVariant&
         return AMF_INVALID_ARG;
     }
 
-    valueOut = amf_int64(paramValue);
-    return AMF_OK;
-}
-
-
-
-
-
-AMF_RESULT ParamConverterFRCSnapshotMode(const std::wstring& value, amf::AMFVariant& valueOut)
-{
-    amf_int64    paramValue = -1;
-    std::wstring uppValue = toUpper(value);
-    if ((uppValue == L"OFF") || (uppValue == L"0"))
-    {
-        paramValue = FRC_SNAPSHOT_OFF;
-    }
-    else if ((uppValue == L"LOAD") || (uppValue == L"1"))
-    {
-        paramValue = FRC_SNAPSHOT_LOAD;
-    }
-    else if ((uppValue == L"STORE") || (uppValue == L"2"))
-    {
-        paramValue = FRC_SNAPSHOT_STORE;
-    }
-    else if ((uppValue == L"TEST") || (uppValue == L"3"))
-    {
-        paramValue = FRC_SNAPSHOT_REGRESSION_TEST;
-    }
-    else if ((uppValue == L"STORE_NO_PADDING") || (uppValue == L"4"))
-    {
-        paramValue = FRC_SNAPSHOT_STORE_NO_PADDING;
-    }
     valueOut = amf_int64(paramValue);
     return AMF_OK;
 }
\ No newline at end of file
